@@ -4,7 +4,9 @@ import time
 from enum import Enum
 
 import kubernetes
-from dagster import DagsterInstance, check
+
+from dagster import DagsterInstance
+from dagster import _check as check
 from dagster.core.storage.pipeline_run import PipelineRunStatus
 
 DEFAULT_WAIT_TIMEOUT = 86400.0  # 1 day
