@@ -1,7 +1,11 @@
+from typing import List, Optional, cast
+
 import kubernetes
 from dagster_k8s.launcher import K8sRunLauncher
 
-from dagster import Field, StringSource, check, executor
+from dagster import Field, IntSource, StringSource
+from dagster import _check as check
+from dagster import executor
 from dagster.core.definitions.executor_definition import multiple_process_executor_requirements
 from dagster.core.errors import DagsterUnmetExecutorRequirementsError
 from dagster.core.events import DagsterEvent, DagsterEventType, EngineEventData, MetadataEntry
@@ -12,7 +16,6 @@ from dagster.core.executor.init import InitExecutorContext
 from dagster.core.executor.step_delegating import StepDelegatingExecutor
 from dagster.core.executor.step_delegating.step_handler import StepHandler
 from dagster.core.executor.step_delegating.step_handler.base import StepHandlerContext
-from dagster.core.types.dagster_type import Optional
 from dagster.utils import frozentags, merge_dicts
 
 from .container_context import K8sContainerContext
@@ -29,8 +32,16 @@ from .utils import delete_job
     name="k8s",
     config_schema=merge_dicts(
         DagsterK8sJobConfig.config_type_job(),
-        {"job_namespace": Field(StringSource, is_required=False)},
-        {"retries": get_retries_config()},
+        {
+            "job_namespace": Field(StringSource, is_required=False),
+            "retries": get_retries_config(),
+            "max_concurrent": Field(
+                IntSource,
+                is_required=False,
+                description="Limit on the number of pods that will run concurrently within the scope "
+                "of a Dagster run. Note that this limit is per run, not global.",
+            ),
+        },
     ),
     requirements=multiple_process_executor_requirements(),
 )
@@ -59,6 +70,11 @@ def k8s_job_executor(init_context: InitExecutorContext) -> Executor:
             env_secrets: ...
             env_vars: ...
             job_image: ... # leave out if using userDeployments
+            max_concurrent: ...
+
+    `max_concurrent` limits the number of pods that will execute concurrently for one run. By default
+    there is no limit- it will maximally parallel as allowed by the DAG. Note that this is not a
+    global limit.
 
     Configuration set on the Kubernetes Jobs and Pods created by the `K8sRunLauncher` will also be
     set on Kubernetes Jobs and Pods created by the `k8s_job_executor`.
@@ -74,26 +90,28 @@ def k8s_job_executor(init_context: InitExecutorContext) -> Executor:
     exc_cfg = init_context.executor_config
 
     k8s_container_context = K8sContainerContext(
-        image_pull_policy=exc_cfg.get("image_pull_policy"),
-        image_pull_secrets=exc_cfg.get("image_pull_secrets"),
-        service_account_name=exc_cfg.get("service_account_name"),
-        env_config_maps=exc_cfg.get("env_config_maps"),
-        env_secrets=exc_cfg.get("env_secrets"),
-        env_vars=exc_cfg.get("env_vars"),
-        volume_mounts=exc_cfg.get("volume_mounts"),
-        volumes=exc_cfg.get("volumes"),
-        labels=exc_cfg.get("labels"),
-        namespace=exc_cfg.get("job_namespace"),
+        image_pull_policy=exc_cfg.get("image_pull_policy"),  # type: ignore
+        image_pull_secrets=exc_cfg.get("image_pull_secrets"),  # type: ignore
+        service_account_name=exc_cfg.get("service_account_name"),  # type: ignore
+        env_config_maps=exc_cfg.get("env_config_maps"),  # type: ignore
+        env_secrets=exc_cfg.get("env_secrets"),  # type: ignore
+        env_vars=exc_cfg.get("env_vars"),  # type: ignore
+        volume_mounts=exc_cfg.get("volume_mounts"),  # type: ignore
+        volumes=exc_cfg.get("volumes"),  # type: ignore
+        labels=exc_cfg.get("labels"),  # type: ignore
+        namespace=exc_cfg.get("job_namespace"),  # type: ignore
+        resources=exc_cfg.get("resources"),  # type: ignore
     )
 
     return StepDelegatingExecutor(
         K8sStepHandler(
-            image=exc_cfg.get("job_image"),
+            image=exc_cfg.get("job_image"),  # type: ignore
             container_context=k8s_container_context,
             load_incluster_config=run_launcher.load_incluster_config,
             kubeconfig_file=run_launcher.kubeconfig_file,
         ),
-        retries=RetryMode.from_config(init_context.executor_config["retries"]),
+        retries=RetryMode.from_config(init_context.executor_config["retries"]),  # type: ignore
+        max_concurrent=check.opt_int_elem(exc_cfg, "max_concurrent"),
         should_verify_step=True,
     )
 
@@ -132,7 +150,8 @@ class K8sStepHandler(StepHandler):
 
     def _get_container_context(self, step_handler_context: StepHandlerContext):
         run_target = K8sContainerContext.create_for_run(
-            step_handler_context.pipeline_run, step_handler_context.instance.run_launcher
+            step_handler_context.pipeline_run,
+            cast(K8sRunLauncher, step_handler_context.instance.run_launcher),
         )
         return run_target.merge(self._executor_container_context)
 
@@ -158,10 +177,11 @@ class K8sStepHandler(StepHandler):
     def launch_step(self, step_handler_context: StepHandlerContext):
         events = []
 
-        assert (
-            len(step_handler_context.execute_step_args.step_keys_to_execute) == 1
-        ), "Launching multiple steps is not currently supported"
-        step_key = step_handler_context.execute_step_args.step_keys_to_execute[0]
+        step_keys_to_execute = cast(
+            List[str], step_handler_context.execute_step_args.step_keys_to_execute
+        )
+        assert len(step_keys_to_execute) == 1, "Launching multiple steps is not currently supported"
+        step_key = step_keys_to_execute[0]
 
         job_name = self._get_k8s_step_job_name(step_handler_context)
         pod_name = job_name
@@ -194,7 +214,7 @@ class K8sStepHandler(StepHandler):
             component="step_worker",
             user_defined_k8s_config=user_defined_k8s_config,
             labels={
-                "dagster/job": step_handler_context.execute_step_args.pipeline_origin.pipeline_name,
+                "dagster/job": step_handler_context.pipeline_run.pipeline_name,
                 "dagster/op": step_key,
                 "dagster/run-id": step_handler_context.execute_step_args.pipeline_run_id,
             },
@@ -203,7 +223,7 @@ class K8sStepHandler(StepHandler):
         events.append(
             DagsterEvent(
                 event_type_value=DagsterEventType.ENGINE_EVENT.value,
-                pipeline_name=step_handler_context.execute_step_args.pipeline_origin.pipeline_name,
+                pipeline_name=step_handler_context.pipeline_run.pipeline_name,
                 step_key=step_key,
                 message=f"Executing step {step_key} in Kubernetes job {job_name}",
                 event_specific_data=EngineEventData(
@@ -220,10 +240,11 @@ class K8sStepHandler(StepHandler):
         return events
 
     def check_step_health(self, step_handler_context: StepHandlerContext):
-        assert (
-            len(step_handler_context.execute_step_args.step_keys_to_execute) == 1
-        ), "Launching multiple steps is not currently supported"
-        step_key = step_handler_context.execute_step_args.step_keys_to_execute[0]
+        step_keys_to_execute = cast(
+            List[str], step_handler_context.execute_step_args.step_keys_to_execute
+        )
+        assert len(step_keys_to_execute) == 1, "Launching multiple steps is not currently supported"
+        step_key = step_keys_to_execute[0]
 
         job_name = self._get_k8s_step_job_name(step_handler_context)
 
@@ -236,7 +257,7 @@ class K8sStepHandler(StepHandler):
             return [
                 DagsterEvent(
                     event_type_value=DagsterEventType.STEP_FAILURE.value,
-                    pipeline_name=step_handler_context.execute_step_args.pipeline_origin.pipeline_name,
+                    pipeline_name=step_handler_context.pipeline_run.pipeline_name,
                     step_key=step_key,
                     message=f"Discovered failed Kubernetes job {job_name} for step {step_key}",
                     event_specific_data=StepFailureData(
@@ -248,9 +269,10 @@ class K8sStepHandler(StepHandler):
         return []
 
     def terminate_step(self, step_handler_context: StepHandlerContext):
-        assert (
-            len(step_handler_context.execute_step_args.step_keys_to_execute) == 1
-        ), "Launching multiple steps is not currently supported"
+        step_keys_to_execute = cast(
+            List[str], step_handler_context.execute_step_args.step_keys_to_execute
+        )
+        assert len(step_keys_to_execute) == 1, "Launching multiple steps is not currently supported"
 
         job_name = self._get_k8s_step_job_name(step_handler_context)
         container_context = self._get_container_context(step_handler_context)
