@@ -1,7 +1,6 @@
 import copy
 import hashlib
 import json
-import os
 import random
 import string
 from collections import namedtuple
@@ -9,12 +8,13 @@ from typing import Any, Dict, List, Optional
 
 import kubernetes
 
+import dagster._check as check
 from dagster import Array, BoolSource, Field, Noneable, StringSource
 from dagster import __version__ as dagster_version
-from dagster import check
 from dagster.config.field_utils import Permissive, Shape
 from dagster.config.validate import validate_config
 from dagster.core.errors import DagsterInvalidConfigError
+from dagster.core.utils import parse_env_var
 from dagster.serdes import whitelist_for_serdes
 from dagster.utils import frozentags, merge_dicts
 
@@ -211,7 +211,7 @@ class DagsterK8sJobConfig(
         "_K8sJobTaskConfig",
         "job_image dagster_home image_pull_policy image_pull_secrets service_account_name "
         "instance_config_map postgres_password_secret env_config_maps env_secrets env_vars "
-        "volume_mounts volumes labels",
+        "volume_mounts volumes labels resources",
     )
 ):
     """Configuration parameters for launching Dagster Jobs on Kubernetes.
@@ -255,6 +255,8 @@ class DagsterK8sJobConfig(
             https://v1-18.docs.kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#volume-v1-core
         labels (Optional[Dict[str, str]]): Additional labels that should be included in the Job's Pod. See:
             https://kubernetes.io/docs/concepts/overview/working-with-objects/labels
+        resources (Optional[Dict[str, Any]]) Compute resource requirements for the container. See:
+            https://kubernetes.io/docs/concepts/configuration/manage-resources-containers/
     """
 
     def __new__(
@@ -272,6 +274,7 @@ class DagsterK8sJobConfig(
         volume_mounts=None,
         volumes=None,
         labels=None,
+        resources=None,
     ):
         return super(DagsterK8sJobConfig, cls).__new__(
             cls,
@@ -300,6 +303,7 @@ class DagsterK8sJobConfig(
                 for volume in check.opt_list_param(volumes, "volumes")
             ],
             labels=check.opt_dict_param(labels, "labels", key_type=str, value_type=str),
+            resources=check.opt_dict_param(resources, "resources", key_type=str),
         )
 
     @classmethod
@@ -412,8 +416,9 @@ class DagsterK8sJobConfig(
             "env_vars": Field(
                 Noneable(Array(str)),
                 is_required=False,
-                description="A list of environment variables to inject into the Job. "
-                "Default: ``[]``. See: "
+                description="A list of environment variables to inject into the Job. Each can be "
+                "of the form KEY=VALUE or just KEY (in which case the value will be pulled from "
+                "the current process). Default: ``[]``. See: "
                 "https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables",
             ),
             "volume_mounts": Field(
@@ -456,6 +461,17 @@ class DagsterK8sJobConfig(
                 description="Labels to apply to all created pods. See: "
                 "https://kubernetes.io/docs/concepts/overview/working-with-objects/labels",
             ),
+            "resources": Field(
+                Noneable(
+                    {
+                        "limits": Field(dict, is_required=False),
+                        "requests": Field(dict, is_required=False),
+                    }
+                ),
+                is_required=False,
+                description="Compute resource requirements for the container. See: "
+                "https://kubernetes.io/docs/concepts/configuration/manage-resources-containers/",
+            ),
         }
 
     @classmethod
@@ -475,7 +491,11 @@ class DagsterK8sJobConfig(
 
     @property
     def env(self) -> List[Dict[str, Optional[str]]]:
-        return [{"name": key, "value": os.getenv(key)} for key in (self.env_vars or [])]
+        parsed_env_vars = [parse_env_var(key) for key in (self.env_vars or [])]
+        return [
+            {"name": parsed_env_var[0], "value": parsed_env_var[1]}
+            for parsed_env_var in parsed_env_vars
+        ]
 
     @property
     def env_from_sources(self) -> List[Dict[str, Any]]:
@@ -607,19 +627,11 @@ def construct_dagster_k8s_job(
 
     user_defined_k8s_volume_mounts = container_config.pop("volume_mounts", [])
 
-    volume_mounts = (
-        [
-            {
-                "name": "dagster-instance",
-                "mount_path": "{dagster_home}/dagster.yaml".format(
-                    dagster_home=job_config.dagster_home
-                ),
-                "sub_path": "dagster.yaml",
-            }
-        ]
-        + job_config.volume_mounts
-        + user_defined_k8s_volume_mounts
-    )
+    user_defined_resources = container_config.pop("resources", {})
+
+    volume_mounts = job_config.volume_mounts + user_defined_k8s_volume_mounts
+
+    resources = user_defined_resources if user_defined_resources else job_config.resources
 
     container_config = merge_dicts(
         container_config,
@@ -631,6 +643,7 @@ def construct_dagster_k8s_job(
             "env": env + job_config.env + additional_k8s_env_vars,
             "env_from": job_config.env_from_sources + additional_k8s_env_from,
             "volume_mounts": volume_mounts,
+            "resources": resources,
         },
     )
 
@@ -638,11 +651,7 @@ def construct_dagster_k8s_job(
 
     user_defined_volumes = pod_spec_config.pop("volumes", [])
 
-    volumes = (
-        [{"name": "dagster-instance", "config_map": {"name": job_config.instance_config_map}}]
-        + job_config.volumes
-        + user_defined_volumes
-    )
+    volumes = job_config.volumes + user_defined_volumes
 
     # If the user has defined custom labels, remove them from the pod_template_spec_metadata
     # key and merge them with the dagster labels
@@ -683,6 +692,9 @@ def construct_dagster_k8s_job(
         {"template": template},
     )
 
+    user_defined_job_metadata = copy.deepcopy(user_defined_k8s_config.job_metadata)
+    user_defined_job_labels = user_defined_job_metadata.pop("labels", {})
+
     job = k8s_model_from_dict(
         kubernetes.client.V1Job,
         merge_dicts(
@@ -691,8 +703,13 @@ def construct_dagster_k8s_job(
                 "api_version": "batch/v1",
                 "kind": "Job",
                 "metadata": merge_dicts(
-                    user_defined_k8s_config.job_metadata,
-                    {"name": job_name, "labels": dagster_labels},
+                    user_defined_job_metadata,
+                    {
+                        "name": job_name,
+                        "labels": merge_dicts(
+                            dagster_labels, user_defined_job_labels, job_config.labels
+                        ),
+                    },
                 ),
                 "spec": job_spec_config,
             },
